@@ -2,8 +2,8 @@ import asyncio
 import contextlib
 import functools
 import logging
-import sys
 import typing
+from contextvars import ContextVar
 from types import TracebackType
 from urllib.parse import SplitResult, parse_qsl, unquote, urlsplit
 
@@ -11,12 +11,7 @@ from sqlalchemy import text
 from sqlalchemy.sql import ClauseElement
 
 from databases.importer import import_from_string
-from databases.interfaces import ConnectionBackend, DatabaseBackend, TransactionBackend
-
-if sys.version_info >= (3, 7):  # pragma: no cover
-    import contextvars as contextvars
-else:  # pragma: no cover
-    import aiocontextvars as contextvars
+from databases.interfaces import DatabaseBackend, Record
 
 try:  # pragma: no cover
     import click
@@ -69,9 +64,7 @@ class Database:
         self._backend = backend_cls(self.url, **self.options)
 
         # Connections are stored as task-local state.
-        self._connection_context = contextvars.ContextVar(
-            "connection_context"
-        )  # type: contextvars.ContextVar
+        self._connection_context = ContextVar("connection_context")  # type: ContextVar
 
         # When `force_rollback=True` is used, we use a single global
         # connection, within a transaction that always rolls back.
@@ -120,7 +113,7 @@ class Database:
             self._global_transaction = None
             self._global_connection = None
         else:
-            self._connection_context = contextvars.ContextVar("connection_context")
+            self._connection_context = ContextVar("connection_context")
 
         await self._backend.disconnect()
         logger.info(
@@ -136,35 +129,41 @@ class Database:
 
     async def __aexit__(
         self,
-        exc_type: typing.Type[BaseException] = None,
-        exc_value: BaseException = None,
-        traceback: TracebackType = None,
+        exc_type: typing.Optional[typing.Type[BaseException]] = None,
+        exc_value: typing.Optional[BaseException] = None,
+        traceback: typing.Optional[TracebackType] = None,
     ) -> None:
         await self.disconnect()
 
     async def fetch_all(
-        self, query: typing.Union[ClauseElement, str], values: dict = None
-    ) -> typing.List[typing.Mapping]:
+        self,
+        query: typing.Union[ClauseElement, str],
+        values: typing.Optional[dict] = None,
+    ) -> typing.List[Record]:
         async with self.connection() as connection:
             return await connection.fetch_all(query, values)
 
     async def fetch_one(
-        self, query: typing.Union[ClauseElement, str], values: dict = None
-    ) -> typing.Optional[typing.Mapping]:
+        self,
+        query: typing.Union[ClauseElement, str],
+        values: typing.Optional[dict] = None,
+    ) -> typing.Optional[Record]:
         async with self.connection() as connection:
             return await connection.fetch_one(query, values)
 
     async def fetch_val(
         self,
         query: typing.Union[ClauseElement, str],
-        values: dict = None,
+        values: typing.Optional[dict] = None,
         column: typing.Any = 0,
     ) -> typing.Any:
         async with self.connection() as connection:
             return await connection.fetch_val(query, values, column=column)
 
     async def execute(
-        self, query: typing.Union[ClauseElement, str], values: dict = None
+        self,
+        query: typing.Union[ClauseElement, str],
+        values: typing.Optional[dict] = None,
     ) -> typing.Any:
         async with self.connection() as connection:
             return await connection.execute(query, values)
@@ -176,17 +175,14 @@ class Database:
             return await connection.execute_many(query, values)
 
     async def iterate(
-        self, query: typing.Union[ClauseElement, str], values: dict = None
+        self,
+        query: typing.Union[ClauseElement, str],
+        values: typing.Optional[dict] = None,
     ) -> typing.AsyncGenerator[typing.Mapping, None]:
         async with self.connection() as connection:
             async for record in connection.iterate(query, values):
                 yield record
 
-    def _new_connection(self) -> "Connection":
-        connection = Connection(self._backend)
-        self._connection_context.set(connection)
-        return connection
-
     def connection(self) -> "Connection":
         if self._global_connection is not None:
             return self._global_connection
@@ -194,23 +190,14 @@ class Database:
         try:
             return self._connection_context.get()
         except LookupError:
-            return self._new_connection()
+            connection = Connection(self._backend)
+            self._connection_context.set(connection)
+            return connection
 
     def transaction(
         self, *, force_rollback: bool = False, **kwargs: typing.Any
     ) -> "Transaction":
-        try:
-            connection = self._connection_context.get()
-            is_root = not connection._transaction_stack
-            if is_root:
-                newcontext = contextvars.copy_context()
-                get_conn = lambda: newcontext.run(self._new_connection)
-            else:
-                get_conn = self.connection
-        except LookupError:
-            get_conn = self.connection
-
-        return Transaction(get_conn, force_rollback=force_rollback, **kwargs)
+        return Transaction(self.connection, force_rollback=force_rollback, **kwargs)
 
     @contextlib.contextmanager
     def force_rollback(self) -> typing.Iterator[None]:
@@ -246,16 +233,16 @@ class Connection:
             try:
                 if self._connection_counter == 1:
                     await self._connection.acquire()
-            except Exception as e:
+            except BaseException as e:
                 self._connection_counter -= 1
                 raise e
         return self
 
     async def __aexit__(
         self,
-        exc_type: typing.Type[BaseException] = None,
-        exc_value: BaseException = None,
-        traceback: TracebackType = None,
+        exc_type: typing.Optional[typing.Type[BaseException]] = None,
+        exc_value: typing.Optional[BaseException] = None,
+        traceback: typing.Optional[TracebackType] = None,
     ) -> None:
         async with self._connection_lock:
             assert self._connection is not None
@@ -264,15 +251,19 @@ class Connection:
                 await self._connection.release()
 
     async def fetch_all(
-        self, query: typing.Union[ClauseElement, str], values: dict = None
-    ) -> typing.List[typing.Mapping]:
+        self,
+        query: typing.Union[ClauseElement, str],
+        values: typing.Optional[dict] = None,
+    ) -> typing.List[Record]:
         built_query = self._build_query(query, values)
         async with self._query_lock:
             return await self._connection.fetch_all(built_query)
 
     async def fetch_one(
-        self, query: typing.Union[ClauseElement, str], values: dict = None
-    ) -> typing.Optional[typing.Mapping]:
+        self,
+        query: typing.Union[ClauseElement, str],
+        values: typing.Optional[dict] = None,
+    ) -> typing.Optional[Record]:
         built_query = self._build_query(query, values)
         async with self._query_lock:
             return await self._connection.fetch_one(built_query)
@@ -280,7 +271,7 @@ class Connection:
     async def fetch_val(
         self,
         query: typing.Union[ClauseElement, str],
-        values: dict = None,
+        values: typing.Optional[dict] = None,
         column: typing.Any = 0,
     ) -> typing.Any:
         built_query = self._build_query(query, values)
@@ -288,7 +279,9 @@ class Connection:
             return await self._connection.fetch_val(built_query, column)
 
     async def execute(
-        self, query: typing.Union[ClauseElement, str], values: dict = None
+        self,
+        query: typing.Union[ClauseElement, str],
+        values: typing.Optional[dict] = None,
     ) -> typing.Any:
         built_query = self._build_query(query, values)
         async with self._query_lock:
@@ -302,7 +295,9 @@ class Connection:
             await self._connection.execute_many(queries)
 
     async def iterate(
-        self, query: typing.Union[ClauseElement, str], values: dict = None
+        self,
+        query: typing.Union[ClauseElement, str],
+        values: typing.Optional[dict] = None,
     ) -> typing.AsyncGenerator[typing.Any, None]:
         built_query = self._build_query(query, values)
         async with self.transaction():
@@ -324,7 +319,7 @@ class Connection:
 
     @staticmethod
     def _build_query(
-        query: typing.Union[ClauseElement, str], values: dict = None
+        query: typing.Union[ClauseElement, str], values: typing.Optional[dict] = None
     ) -> ClauseElement:
         if isinstance(query, str):
             query = text(query)
@@ -336,6 +331,9 @@ class Connection:
         return query
 
 
+_CallableType = typing.TypeVar("_CallableType", bound=typing.Callable)
+
+
 class Transaction:
     def __init__(
         self,
@@ -356,9 +354,9 @@ class Transaction:
 
     async def __aexit__(
         self,
-        exc_type: typing.Type[BaseException] = None,
-        exc_value: BaseException = None,
-        traceback: TracebackType = None,
+        exc_type: typing.Optional[typing.Type[BaseException]] = None,
+        exc_value: typing.Optional[BaseException] = None,
+        traceback: typing.Optional[TracebackType] = None,
     ) -> None:
         """
         Called when exiting `async with database.transaction()`
@@ -368,13 +366,13 @@ class Transaction:
         else:
             await self.commit()
 
-    def __await__(self) -> typing.Generator:
+    def __await__(self) -> typing.Generator[None, None, "Transaction"]:
         """
         Called if using the low-level `transaction = await database.transaction()`
         """
         return self.start().__await__()
 
-    def __call__(self, func: typing.Callable) -> typing.Callable:
+    def __call__(self, func: _CallableType) -> _CallableType:
         """
         Called if using `@database.transaction()` as a decorator.
         """
@@ -384,7 +382,7 @@ class Transaction:
             async with self:
                 return await func(*args, **kwargs)
 
-        return wrapper
+        return wrapper  # type: ignore
 
     async def start(self) -> "Transaction":
         self._connection = self._connection_callable()
